<?php
 
define('MAILOUT_SEND_FIRST', 'First'); // We actually are not using this button, just setting the form state.
define('MAILOUT_SEND_NEXT', 'Next');
define('MAILOUT_SEND_REVIEW', 'Update');
define('MAILOUT_SEND_READY', 'Ready to Send');
define('MAILOUT_SEND_SEND', 'Send');
define('MAILOUT_SEND_BACK', 'Back');
define('MAILOUT_SEND_TEST', 'Send test email');

/**
 * callback for mailout/send
 *
 * @return $form 
 */
function mailout_send() {
  return drupal_get_form('mailout_send_form');
}

/**
 * multi step form for sending a mailout
 *
 * @param unknown_type $form_values
 * @return $form
 */
function mailout_send_form($form_values = NULL) {
//  dsm($form_values);

  $form = array (
    '#multistep'  => TRUE,
    '#redirect'   => FALSE,
    '#tree' => TRUE,
  );
  if ($form_values) {
    if ($form_values['op'] == t(MAILOUT_SEND_NEXT) || $form_values['op'] == t(MAILOUT_SEND_BACK) || $form_values['op'] == t(MAILOUT_SEND_TEST) || $form_values['op'] == t(MAILOUT_SEND_TEST)) {
      // We've at least done the first page (selecting the template).
      if (!$form_values['send_to']['init']) {
        // We haven't done the user selection page yet.
        $action = t(MAILOUT_SEND_NEXT);
      }
      else {
        $action = t(MAILOUT_SEND_REVIEW);
      }
    }
    elseif ($form_values['op'] == t(MAILOUT_SEND_READY)) {
      $action = t(MAILOUT_SEND_READY);
    }
    elseif ($form_values['op'] == t(MAILOUT_SEND_SEND)) {
      $action = t(MAILOUT_SEND_SEND);
      $form['#redirect'] = NULL;
    }
// Not working.
//    if (isset($form_values['state']['mtid']) && $form_values['state']['mtid'] != $form_values['template']['mtid']) {
//      $mtid_changed = TRUE;
//    }
  }
  else {
    // No form values yet, so this is the first page to select the template.
    $action = t(MAILOUT_SEND_FIRST);
    $form['state']['mtid'] = array('#type' => 'hidden', '#value' => 0);
  }
    
  // TEMPLATE PREP

  // Get all mailout templates.
  $options_templates = array('0' => " - No Template - ");
  $all_templates = mailout_get_templates();
  $current_template = (object)array('mtid' => 0, 'subject' => 'News from '. variable_get('site_name', 'my site.'), 'email' => '"'. variable_get('site_name', 'my site.') .'" <'. variable_get('site_mail', '') .'>', 'opt_out' => 1, 'stat_track' => 1, 'header' => '', 'body' => '', 'footer' => '', 'style' => '');

  foreach ($all_templates AS $id => $template) {
    $options_templates[$id] = $template->subject;
    if ($id == $form_values['template']['mtid']) {
      $current_template = $template;
    }
  }

  // FIRST PAGE?
  
  if ($action == t(MAILOUT_SEND_FIRST)) {
    // Basic template question for the first page.
    $form['template']['mtid'] = array(
      '#type' => 'select',
      '#title' => t('Template'),
      '#options' => $options_templates,
      '#default_value' => $current_template->mtid,
      '#description' => t('Choose from one of the available templates.'),
    );  
    if (user_access("administer mailout_templates")) {
      $form['template']['mtid']['#description'] .= " Or create a " . l(t('new template'), 'admin/content/mailout/templates/new') .'.'; 
    }
    
    $form['next'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_NEXT),
    );
    // Page one.
    return $form;
  }

  // SEND TO -- Prepare this now, and use for the second page perhaps, or we'll build the rest
  // of the form a bit later.

  // Fieldset
  $form_send_to = array('#type' => 'fieldset', '#title' => 'Recipients', '#collapsible' => TRUE, '#collapsed' => (($action == t(MAILOUT_SEND_NEXT)) ? FALSE : TRUE));
  $this_bit = $form_values['send_to'];
  
  // Build Mailout list options.
  $options_lists = array();
  foreach (mailout_get_lists() AS $list) {
    $options_lists[$list->id] = $list->list_name;
  }
  asort($options_lists);

  // This field allows sending of mailout to mailout lists
  if(count($options_lists)) {
    $form_send_to['lists'] = array(
     '#type' => 'checkboxes',
     '#title' => t('Send to Mailout List(s)'),
     '#options' => $options_lists,
     '#description' => t("Send mailout to one or more mailout lists."),        
     '#required' => FALSE,
    );
  }

  // Build local users options.
  $options_roles = user_roles(TRUE);
  $options_roles[2] = 'All users'; // 'authenticated user' makes no sense.
  ksort($options_roles);

  $form_send_to['init']['#type'] = 'hidden';
  $form_send_to['init']['#value'] = TRUE;

  // This field allows sending of mailout to all users of selected role or all users.
  $form_send_to['roles'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Send to Role(s)'),
    '#options' => $options_roles,
    '#description' => t("Send mailout to selected user roles."),      
    '#required' => FALSE,
  );
  
  // Build options list of all Organic Groups.
  if (module_exists('og')) {
    $options_groups = og_all_groups_options();
    // This field allows sending of mailout to mailout lists.
    if(count($group_array)) {
      $form_send_to['groups'] = array(
       '#type' => 'checkboxes',
       '#title' => t('Send to Organic Group(s)'),
       '#options' => $options_groups,
       '#description' => t("Send mailout to one or more groups."),        
      );          
    }      
  }
  
  // Build Mailout list options.
  $form_send_to['email'] = array(
    '#type' => 'textarea',
    '#title' => t('Email addresses'),
    '#rows' => 4,
    '#value' => mailout_get_addresses_from_text($this_bit['email'] ."\n". $this_bit['user'], TRUE),
    '#description' => t("Dump email addresses here. Comma delimited, space delimited, or one per line."),
    '#required' => FALSE,
  );

  $form_send_to['user'] = array(
    '#type' => 'textfield',
    '#title' => t('Any user'),
    '#autocomplete_path' => 'admin/content/mailout/autocomplete',
    '#default_value' => '',
    '#value' => '',
    '#description' => t('Type a user or email address. The results will be Mailout subscribers and/or CMS users.'),
  );
  
  // SECOND PAGE?
  
  if ($action == t(MAILOUT_SEND_NEXT)) {
    // User questions for the second page.
    $form['template']['mtid'] = array(
      '#type' => 'hidden',
      '#value' => $current_template->mtid,
    );
    $form['send_to'] = $form_send_to;
    $form['next'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_NEXT),
    );

    // Page two.
    return $form;
  }

  // MAIN FORM
  
  // Fieldset
  $form['template'] = array('#type' => 'fieldset', '#title' => 'Template', '#collapsible' => TRUE, '#collapsed' => FALSE  ,);
  $this_bit = $form_values['template'];

  $form['template']['mtid'] = array(
    '#type' => 'select',
    '#title' => t('Template'),
    '#options' => $options_templates,
    '#value' => $current_template->mtid,
    '#description' => t('Changing the template will cause some values to reset (except the body of the email).'),
  );
  $form['template']['html'] = array(
    '#type' => 'select',
    '#title' => t('Format'),
    '#options' => array('1' => 'User preference', '0' => 'Plain text only'),
    '#value' => ($this_bit['html']) ? $this_bit['html'] : 1,
    '#description' => t('Should this email be sent as plain text only (regardless of what each subscriber prefers)? If you choose plain, you will need to use the \'Plain text version\' area to write your newsletter.'),
  );
  $form['template']['header'] = array(
    '#type' => 'textarea',
    '#title' => t('E-mail header'),
    '#rows' => 10,
    '#value' => ($mtid_changed || !isset($this_bit['header'])) ? $current_template->header : $this_bit['header'],
  );
  $form['template']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('E-mail body'),
    '#description' => t('This will be inserted between the header and footer of the sent email.'),
    '#rows' => 10,
    '#default_value' => $current_template->body,
  );
  $form['template']['footer'] = array(
    '#type' => 'textarea',
    '#title' => t('E-mail footer'),
    '#rows' => 10,
    '#value' => ($mtid_changed || !isset($this_bit['footer'])) ? $current_template->footer : $this_bit['footer'],
  );
  preg_match_all('/\s*(.*)\s*\{\s*(.*)\s*\}/sU', $current_template->style, $matches, PREG_SET_ORDER);
  $style_msg = '';
  foreach ($matches AS $key => $match) {
    $tag = trim($match[1]);
    $style = str_replace(array("\n", "\r"), ' ', $match[2]);
    $style_msg .= "<$tag style=\"$style\">&lt;$tag&gt;Example&lt;/$tag&gt;</$tag> \n";
  }
  $style_msg = theme('fieldset', array('#title' => 'Style preview', '#collapsible' => FALSE, '#value' => $style_msg,));
  $form['template']['style'] = array(
    '#type' => 'textarea',
    '#title' => t('Style'),
    '#description' => t('All tags without an inline style will have these styles applied inline.') . $style_msg,
    '#rows' => 6,
    '#default_value' => $current_template->style,
  );  
  $form['template']['plain'] = array(
    '#type' => 'textarea',
    '#title' => t('Plain text version'),
    '#description' => t('Please enter a plain text version of your mailout. This will be used for subscribers who prefer not to receive HTML emails.'),
    '#rows' => 5,
  );

  // USER

  $form['send_to'] = $form_send_to;

  // TEST

  // Fieldset
  $form['test'] = array('#description' => 'Here you can test a mailout, to see how it will looks in your inbox.', '#type' => 'fieldset', '#title' => 'Test Mailout', '#collapsible' => TRUE, '#collapsed' => TRUE,);

  $preview = array('#value' => $current_template->header .'<br />'. $this_bit['body']['#default_value'] .'<br />'. $current_template->footer,);  

  $form['test']['email'] = array(
    '#type' => 'textfield',
    '#title' => t('Send test to'),
    '#autocomplete_path' => 'admin/content/mailout/autocomplete',
    '#default_value' => '',
    '#description' => t('Type any user or email address. If the email matches a system address, you can choose it from the list.'),
  );
  $form['test']['submit'] = array(
    '#type' => 'submit',
    '#value' => t(MAILOUT_SEND_TEST),
  );

  // FINAL OPTIONS

  // Fieldset
  $form['options'] = array('#type' => 'fieldset', '#title' => 'Options', '#collapsible' => TRUE, '#collapsed' => FALSE,);
  $this_bit = $form_values['options'];

  $form['options']['send_from'] = array(
    '#type'=> 'textfield',
    '#title' => t('E-mail from this address'),
    '#value' => (!isset($this_bit['send_from'])) ? $current_template->email : $this_bit['send_from'],
    '#required' => TRUE,
  );  
  $form['options']['subject'] = array(
    '#type'=> 'textfield',
    '#title' => t('E-mail subject'),
    '#description' => t('Subject of the email to be sent.'),
    '#value' => (!isset($this_bit['subject'])) ? $current_template->subject : $this_bit['subject'],
    '#required' => TRUE,
  );

  $dir = file_directory_path() .'/mailout/mtid-'. $current_template->mtid;
  $found_attachments = file_scan_directory($dir, '.*');
  $dir = file_directory_path() .'/mailout/mtid-all';
  $found_attachments += file_scan_directory($dir, '.*');
  foreach($found_attachments AS $file) {
    $attachment_options[$file->filename] = $file->basename;
  }
  $form['options']['attachments'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Attach files'),
    '#description' => t("Check the files that should be attached to the mailout."),
    '#options' => $attachment_options,
  );
  $form['options']['schedule_date'] = array(
    '#type' => 'date',
    '#title' => t('Scheduled Date'),
    '#description' => t("Schedule mailout to be sent at some date in the future."),
    '#value' => $this_bit['schedule_date'],
  );
  $form['options']['stat_track'] = array(
    '#type' => 'select',
    '#title' => t('Track Statistics'),
    '#options' => array('0' => 'No', '1' => 'Yes'),
    '#value' => ($mtid_changed || !isset($this_bit['stat_track'])) ? $current_template->stat_track : $this_bit['stat_track'],
    '#description' => t('Should a statistics tracking image be embeded in emails? This functionality will only work if sending an email in HTML format and the user allows images to be displayed in their email client.'),
  );
  $form['options']['opt_out'] = array(
    '#type' => 'select',
    '#title' => t('Opt Out Text'),
    '#options' => array('0' => 'No', '1' => 'Yes'),
    '#value' => ($mtid_changed || !isset($this_bit['opt_out'])) ? $current_template->opt_out : $this_bit['opt_out'],
    '#description' => t('Should an opt out link be appended to the bottom of emails?'),
  );

  // Allows form alter to correct some wysiwyg annoyances.
  $form['template']['mailout_wysiwyg_suppress'] = array(
    '#type' => 'hidden',
    '#value' => FALSE,
  );

  if ($action == t(MAILOUT_SEND_READY)) {
    // Hide most of the form. Very cheap way to do this.
    // At the time considered best way, rather than switching elements to hidden or some other technique.

    $form['template']['#title'] = '';
    $form['template']['#collapsible'] = TRUE;
    $form['template']['#collapsed'] = TRUE;
    $form['template']['#weight'] = 100;

    // Allows form alter to correct some wysiwyg annoyances.
    $form['template']['mailout_wysiwyg_suppress'] = array(
      '#type' => 'hidden',
      '#value' => TRUE,
    );
    
    // Need this to work so don't hide this right now.

    $form['send_to']['#title'] = '';
    $form['send_to']['#collapsible'] = TRUE;
    $form['send_to']['#collapsed'] = TRUE;
    $form['send_to']['#weight'] = 101;
    $form['test']['#title'] = '';
    $form['test']['#collapsible'] = TRUE;
    $form['test']['#collapsed'] = TRUE;
    $form['test']['#weight'] = 102;

    $form['review'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_BACK),
    );
    $form['send'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_SEND),
    );

    $all_mail = _mailout_send_collate_emails($form_values['send_to']);
    if (count($all_mail) > 5000) {
      $all_mail_msg = theme('fieldset', array('#title' => t('Verify email addresses'), '#collapsible' => TRUE, '#collapsed' => FALSE, '#children' => "<p>Too many addresses to list (over 5000).</p>"));
    }
    else {
      $all_mail_msg = theme('fieldset', array('#title' => t('Verify email addresses'), '#collapsible' => TRUE, '#collapsed' => FALSE, '#children' => '<pre>'. implode ("\n", array_keys($all_mail)) ."</pre>"));
    }
      
    $form['check_users'] = array(
      '#type' => 'markup',
      '#value' =>  $all_mail_msg,
    );

  }
  else {
    $form['options']['#collapsed'] = TRUE;
    $form['back'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_REVIEW),
    );
    $form['send'] = array(
      '#type' => 'submit',
      '#value' => t(MAILOUT_SEND_READY),
    );
  }

  return $form;
}

function mailout_send_form_validate($form_id, $form_values) {

  if ($form_values['op'] == t(MAILOUT_SEND_NEXT) && !isset($form_values['send_to']['init'])) {
    // Currently just selecting template here, no validation.
    return;
  }

  if (!count(_mailout_send_collate_emails($form_values['send_to']))) {
// Not working.    
//    drupal_set_message(t('Your Recipient choices haven\'t specified any recipients'), 'warning');
  }

// Not working.
//  if ($form_values['state']['mtid'] != $form_values['template']['mtid']) {
    // Template has changed, issue a warning.
//    drupal_set_message('New template has possibly over-ridden the \'E-mail header\' and the \'E-mail footer\'. (See the Template sections.)', 'warning');
//  }

  if ($form_values['op'] == t(MAILOUT_SEND_REVIEW) || $form_values['op'] == t(MAILOUT_SEND_BACK)) {
    // Nothing to validate here, yet.
    return;
  }

  if ($form_values['op'] == t(MAILOUT_SEND_TEST)) {
    if (!$form_values['test']['email']) {
      form_set_error('test][email', 'No test email is set.');
    }
    else {
      mailout_sendmail($form_values, TRUE);
    }
    return;
  }

  if ($form_values['op'] == t(MAILOUT_SEND_READY)) {
    // Once the user is ready to send, we check some values.
    // Not sure what yet.
    return;
  }
}

function _mailout_send_collate_emails($form_values) {
  $emails = array();
  $emails = array_merge($emails, mailout_get_addresses_from_roles($form_values['roles'], FALSE));
  $emails = array_merge($emails, mailout_get_addresses_from_lists($form_values['lists'], FALSE));  
  $emails = array_merge($emails, mailout_get_addresses_from_text($form_values['email'], FALSE));
  ksort($emails);
  return $emails;
}

function mailout_send_form_submit($form_id, $form_values) {
  if ($form_values['op'] != t(MAILOUT_SEND_SEND)) {
    return FALSE;
  }
  // Ready to schedule mail for sending.
  $mlid = mailout_sendmail($form_values);
  $form['#redirect'] = NULL;
  drupal_goto('admin/content/mailout/stats/summary/'. $mlid);
}

/**
 * implementation of hook_submit()
 * used for implementation of multi step form
 *
 * @param unknown_type $form_id
 * @param unknown_type $form_values
 */
function mailout_sendmail($form_values, $test = FALSE) {

  // Build the email headers and serialize.
  $email_headers = array();

  $email_body = $form_values['template']['header'];
  $email_body .= $form_values['template']['body'];
  $email_body .= $form_values['template']['footer'];

  preg_match_all('/\s*(.*)\s*\{\s*(.*)\s*\}/sU', $form_values['template']['style'], $matches, PREG_SET_ORDER);
  foreach ($matches AS $key => $match) {
    $tag = trim($match[1]);
    $style = trim(str_replace(array("\n", "\r"), ' ', $match[2]));
    // This matches all un-attributed tags. Need a way to match <a href="blah"> without overriding inline style for a tag.
    // Basically it would be nice to detect a "style" attribute already in an element in any situation, but not sure how to do this yet.
    if ($tag == 'a') {
      $email_body = str_replace("<$tag ", "<$tag style=\"$style\" ", $email_body); 
    }
    else {
      $email_body = str_replace("<$tag>", "<$tag style=\"$style\">", $email_body); 
    }
  }  
  $plain = check_plain($form_values['template']['plain']);

  $plain_attachments = '';
  $attachments = array();  
  if (isset($form_values['options']['attachments']) && count($form_values['options']['attachments'])) {
    foreach($form_values['options']['attachments'] AS $filename => $include) {
      if ($filename && $filename == $include) {
        // Make this look like 
        $info = pathinfo($include);
        $info['filepath'] = $include; 
        // Boy, this will need work.
        // I don't want to use mime_content_type() or have deps on fileinfo.
        // And since I chose to upload without file.inc/upload.module, I don't know how else to sensibly get the mimetype.
        // Will refactor later, maybe, right now it can stick out like a sore thumb.
        $mimetypes = array(
          'jpeg' => 'image/jpeg',
          'jpg' => 'image/jpeg',
          'gif' => 'image/gif',
          'png' => 'image/png',
          'bmp' => 'image/bmp',
          'wav' => 'audio/wav',
          'ttf' => 'font/ttf',
          'tiff' => 'image/tiff',
          'doc' => 'text/vnd.ms-word',
          'mpg' => 'video/mpeg',
          'mpeg' => 'video/mpeg',
          'html' => 'text/plain',
          'htm' => 'text/plain',
          'mov' => 'video/quicktime',
          'xls' => 'application/vnd.ms-excel',
        );
        $info['filemime'] = $mimetypes[strtolower($info['extension'])];
        $attachments[] = $info;
        $plain_attachments .= "\n". url($include, NULL, NULL, TRUE);
      }
    }
  }

  // Prepare the body as plain so we can muss with it.
  if (strlen(trim($plain)) < 1) {
    include_once drupal_get_path('module', 'mimemail') .'/mimemail.inc';
    $plain = mimemail_html_to_text($email_body, TRUE);
  }

  // Add attachments as links at the bottom of the email.
  if ($plain_attachments) {
    $plain .= "\n\nFile Attachments:". $plain_attachments;
  }
  
  if ($test) {
    mimemail($form_values['options']['send_from'], $form_values['test']['email'], t('(Test HTML) ') . $form_values['options']['subject'], $email_body, FALSE, array(), $plain, $attachments);
    mimemail($form_values['options']['send_from'], $form_values['test']['email'], t('(Test Plain) ') . $form_values['options']['subject'], $email_body, TRUE, array(), $plain, $attachments);
    drupal_set_message(t('An HTML and a Plain test have been sent to ') . $form_values['test']['email']); 
    return;
  }
  
  $address_list = _mailout_send_collate_emails($form_values['send_to']);
/*
  foreach ($address_list AS $key => $value) {
    if (!$form_values['check_users'][$key]) {
      unset($all_mail[$key]);
    }
  }
*/  
  // Insert db record for send.
  db_query(
    "INSERT INTO {mailout_log} (mtid, email_subject, email_from, email_body, email_plain, attachments, html, opt_out, stat_track, scheduled) VALUES (%d, '%s', '%s', '%s', '%s', '%s', %d, %d, %d, %d)",
    $form_values['template']['mtid'], $form_values['options']['subject'], $form_values['options']['send_from'], $email_body, $plain, serialize($attachments), $form_values['template']['html'], $form_values['options']['opt_out'], $form_values['options']['stat_track'], $form_values['options']['scheduled_timestamp']
  );

  // Get log_id just inserted.
  $mlid = db_result(db_query('SELECT MAX(mlid) as mlid FROM {mailout_log}'));
  
  $message_count = 0;
  foreach ($address_list AS $key => $data) {
    // Insert db record for each insert
    db_query("INSERT INTO {mailout_message_log} (mlid, email_to, name, html) VALUES (%d, '%s', '%s', %d)", $mlid, $data['email'], $data['name'], $data['html']);
    $message_count++; 
  }
  
  // Output details
  drupal_set_message("Mailout scheduled to send to $message_count addresses on next cron run.<br>".l('Click here to run cron now', 'admin/content/mailout/stats/cron/'. $mlid));
  return $mlid;
}


/**
 * Implementation of hook_cron()
 * send emails that need to be sent
 */
function mailout_send_cron($list_id = NULL) {
  $time = time();
  $count = 0; $errors = array();
  $max_seconds = variable_get('mailout_batch_max_seconds', 25);
  $max_emails = variable_get('mailout_batch_max_emails', 3);
  
  // Get mailout e-mails to be sent
  if ($list_id) {
    $result = db_query('SELECT ml.mmid, l.mlid, ml.email_to, ml.name, ml.html  AS user_html, l.html, l.stat_track, l.opt_out, l.email_headers, l.email_subject, l.email_body, l.email_plain, l.attachments, l.email_from FROM {mailout_message_log} AS ml JOIN {mailout_log} AS l ON ml.mlid = l.mlid WHERE ml.mlid = %d AND sent = 0 AND l.scheduled < %d ', $list_id, time());
  }
  else {
    $result = db_query('SELECT ml.mmid, l.mlid, ml.email_to, ml.name, ml.html  AS user_html, l.html, l.stat_track, l.opt_out, l.email_headers, l.email_subject, l.email_body, l.email_plain, l.attachments, l.email_from FROM {mailout_message_log} AS ml JOIN {mailout_log} AS l ON ml.mlid = l.mlid WHERE sent = 0 AND l.scheduled < %d ', time());
  }
  
  while ((time() - $time) < $max_seconds && ($count + count($errors) < $max_emails) && $data = db_fetch_array($result)) {    
    $timestamp = time();
    $hash = mailout_hash($data['mmid'], $timestamp);
    $final_email_body = $data['email_body'];
    $final_plain = $data['email_plain'];
    
    // Add opt out / subscriptions link.
    if ($data['opt_out'] > 0) {
      $url = url('mailout/subscriptions/'. $data['mmid'] .'/'. $timestamp .'/'. $hash, NULL, NULL, TRUE);
      $final_email_body .= theme('mailout_subscriptions_link', $data['email_to'], $url, TRUE);
      $final_plain .= theme('mailout_subscriptions_link', $data['email_to'], $url, FALSE);
    }

    // Add custom tracking image to each email.
    if ($data['stat_track'] && $data['html']) {
      $final_email_body .= "\n<img src=\"". url('mailout/track/'. $data['mmid'] .'/'. $timestamp .'/'. $hash .'/image.gif', NULL, NULL, TRUE) .'" width="0" height="0" />';
    }

    // Send mail
    if (!$data['user_html'] || !$data['html']) {
      // If either the user or the email is plain text, then send in plain text.
      $success = mimemail($data['email_from'], $data['email_to'], $data['email_subject'], $final_email_body, TRUE, array(), $final_plain);
    }
    else {
      $success = mimemail($data['email_from'], $data['email_to'], $data['email_subject'], $final_email_body, FALSE, array(), '', unserialize($data['attachments']));
    }

    // Update the log is we've sent OK.
    if ($success) {
      $count++;
      db_query('UPDATE {mailout_message_log} SET sent = 1 WHERE mmid = %d', $data['mmid']);
    }
    else {
      $errors[] = l($data['email_to'], mailout_list_subscription_path($data['email_to']));
    }
  }
  if ($list_id) {
    drupal_set_message($count .' emails sent in '. ceil(time() - $time) . ' seconds.');
    if (count($errors)) {
      drupal_set_message(t('<p>The following emails could not be processed:</p>') . theme('item_list', $errors), 'warning');
    }
    // This is run through admin pages, so return to the stats page of the list being sent.
    drupal_goto('admin/content/mailout/stats/summary/'. $list_id);
  }
}

/**
 * Theme function to theme the opt out / subscriptions text appended to the end of emails.
 *
 * @param $email - email address for current email.
 * @param $url - working url for callback from unauthenticated user.
 * @param $html_email - 1 if email is being sent as HTML, 0 for plain text.
 */
function theme_mailout_subscriptions_link($email, $url, $html_email) {
  if ($html_email) {
    $link_text = "\n". '<p><a href="'. $url .'">'. t('Manage subscriptions for @email.', array('@email' => $email)) ."</a></p>\n"; 
  }
  else {
    $link_text = "\n\n". t('Please paste this link into your browser to manage subscriptions for @email', array('@email' => $email)) .":\n". $url ."\n";
  }
  return $link_text;
}


